//             TECHNOLOGICAL ARTS
//              +1(416) 963-8996 
//	        www.technologicalarts.ca

#include <hidef.h>      /* common defines and macros */
#include <mc9s12dp512.h>     /* derivative information */
#include <Delay.h>

#define RS_D 0x40	  // RS data for LCD
#define RS_I 0x00	  // RS instruction for LCD

#define EN_H 0x80	  // Enable high for LCD
#define EN_L 0x2F	  // Enable low for LCD

#define LCDLENGTH 16	  // 16 character per line LCD

#define DELAY1 (200000)   // DELAY1 is of 4.1 mSec Approx. Used for LCD initialisation.
#define DELAY2 (5000)     // DELAY2 is of 100 uSec Approx.
#define DELAY3 (2000)     // DELAY3 is of 40 uSec Approx.

#define DELAY4 (5)        // DELAY4 is of .1 uSec Approx.
			                    // provide setup time for 74HC595
			                    // Shift Register

#define DELAY5 (20000)    // DELAY5 is used for KeyPad debounce delay
                          // This delay can be varied depending upon
			                    // how keypad is behaving when you press a key.

#define SS_H 0x80	  // Data Latch High for 74HC595
#define SS_L 0x7F	  // Data Latch Low for 74HC595


int display_Count;  // keep track of how many times keypad is pressed.
			              // After every 16 time this count is set to zero
			              // and LCD start displaying charater from beginning
			              // of line 4.

//**************************************
// Send byte on SPI port
//**************************************
void data_send_SPI(unsigned char data)
{
	unsigned char status, data2;

	SPI0DR=data;	      // send data on SPI port

	while(SPI0SR==0)    // if zero SPI Data register is not empty
	{
	};

	status=SPI0SR;      // read Status and Data Register
	data2= SPI0DR;
        
	Delay(DELAY4);	    // provide setup time for 74HC595
			                // Shift Register
 

 	PTS|=SS_H;  	      // set Data Latch High 
 	PTS&=SS_L;  	      // set Data Latch Low 
}

//**************************************
// Send data to LCD
//**************************************
void Lcd_Controlword(unsigned char data,unsigned char rs) 
{
 
 PTP=EN_L|rs;		    // EN_L and rs will set or reset 
			              // depending on data or instruction
 
 data_send_SPI(data);	  // send data to SPI port


// Delay(5);
// PTS|=SS_H;  
// PTS&=SS_L;  
 
 PTP|=EN_H|rs;		  // EN_L and rs will set or reset 
 PTP=EN_L|rs;		    // depending on data or instruction 
 
 Delay(DELAY3);		  // 40 usec delay for LCD after
			              // each command, to perform its 
			              // internal operation.
}



// *************************************
// This subroutine is intialise the Lcd 
// *************************************

void Lcd_Initialization(void)
{

  Lcd_Controlword(0x30,RS_I);    // funtion set once
  Delay(DELAY1);                 // delay of 4.1 msec
  
  Lcd_Controlword(0x30,RS_I);    // funtion set twice
  Delay(DELAY2);                 // delay of 100usec

  Lcd_Controlword(0x30,RS_I);    // funtion set thrice
     

  Lcd_Controlword(0x38,RS_I);    // last function set: 8-bit mode, 2 lines, 5x7 matrix
  
  Lcd_Controlword(0x0C,RS_I);    // display on, cursor off, blink off
  
  Lcd_Controlword(0x01,RS_I);    // display clear
  
  Lcd_Controlword(0x06,RS_I);    // cursor auto-increment, disable display shift
  
}

 //**************************************************************************
//                             Write Data on LCD, 
// This subroutine will assign unique address to each line and display on LCD
//***************************************************************************
void Lcd_Write(char *string, int line)
{
  int charCount;
	unsigned char address;
  switch(line) 
    {
      case 1:   address = 0x80;   // line 1 address
                break;
      
      case 2:   address = 0xC0;   // line 2 address
                break;
      
      case 3:   address = 0x90;   // line 3 address
                break;
      
      case 4:   address = 0xD0;   // line 4 address
                break;
      default:  break;
    }   

	
	Lcd_Controlword( address,RS_I);  // rs=0 for instruction
	
	for( charCount = 0; charCount < LCDLENGTH; charCount++)
	  {
		  Lcd_Controlword( string[charCount],RS_D); // rs=1 for data
	  }
} 

//*************************************************
// Stores the beginning display information for LCD
//*************************************************
void Lcd_Display(void) 
{

  Lcd_Write("  TECHNOLOGICAL ",1);  	// line 1
  Lcd_Write("      ARTS      ",2);  	// line 2
  Lcd_Write("DKKI KEYPAD DEMO",3);  	// line 3
  Lcd_Write("   Hello World  ",4);  	// line 4

  Lcd_Controlword( 0xD0,RS_I);      	// set beginning address for keypad letters
}



//*************************************************************************************
// This subroutine will keep track of keys pressed on the keypad. After 16 keys pressed
//          line 4 will clear and LCD address is set to the beginning of Line 4.
//*************************************************************************************
void display_keypad(char x)

{     
    Delay(DELAY5);  		    	        // debounce delay
    if(display_Count==LCDLENGTH)    	// check for 16 chacters pressed from KeyPad
       {
        display_Count=0;	    	      // set count to zero
    	Lcd_Write("                ",4);// clear line 4
        Lcd_Controlword( 0xD0,RS_I);  // set starting address for line 4
       }
     display_Count++;			            // incremented after every keypressed 

    Lcd_Controlword(x,RS_D);  		// display key pressed.
}




